
import sys
from helpers import install_packages, juju_log

try:
    import MySQLdb
except:
    install_packages(['python-mysqldb'])
    import MySQLdb


class db:
    def __init__(self, db_user, db_host, db_passwd, db_name):
        self.db_user = db_user
        self.db_host = db_host
        self.db_password = db_passwd
        self.db_name = db_name

        try:
            self.connection = MySQLdb.connect(user=db_user, host=db_host, passwd=db_passwd, db=db_name)
        except MySQLdb.Error, e:
            try:
                juju_log('ERROR', "Could not connect to database '%s'. MySQL Error [%d]: %s" % (db_name, e.args[0], e.args[1]))
            except IndexError:
                juju_log('ERROR', "Could not connect to database '%s'. MySQL Error: %s" % (db_name, str(e)))
            return None

    def close_db(self):
        """ Close connection to database """
        return self.connection.close()

    def commit(self):
        """ Commit transaction """
        return self.connection.commit()

    def database_exists(self, db_name):
        """ Check database existance by name """
        cursor = self.connection.cursor()
        try:
            cursor.execute('SHOW DATABASES LIKE "{}"'.format(db_name))
            databases = [i[0] for i in cursor.fetchall()]
        finally:
            cursor.close()
        return db_name in databases

    def table_exists(self, table_name):
        """ Check table existance by name """
        cursor = self.connection.cursor()
        try:
            cursor.execute('SHOW TABLES LIKE "{}"'.format(table_name))
            tables = [i[0] for i in cursor.fetchall()]
        finally:
            cursor.close()
        return table_name in tables

    def load_from_file(self, filename):
        """ Perform SQL queries from file """
        try:
            f = open(filename, 'rb')
            sql_data = f.read()
            f.close()
        except IOError as e:
            juju_log('ERROR', "I/O error({}): {}".format(e.errno, e.strerror))
            return False

        cursor = self.connection.cursor()

        try:
            cursor.execute(sql_data)
        except MySQLdb.Error, e:
            try:
                juju_log('ERROR', "Could not execute %s SQL file. MySQL Error [%d]: %s" % (filename, e.args[0], e.args[1]))
            except IndexError:
                juju_log('ERROR', "Could not execute %s SQL file. MySQL Error: %s" % (filename, str(e)))

            cursor.close()
            return False

        cursor.close()
        self.commit()

        return True

    def db_fetch_one(self, query, fatal=True):
        """ Select only one value """
        cursor = self.connection.cursor()

        result = None

        try:
            cursor.execute(query)
            data = cursor.fetchone()
        except MySQLdb.Error, e:
            try:
                juju_log('ERROR', "Could not perform query. MySQL Error [%d]: %s" % (e.args[0], e.args[1]))
            except IndexError:
                juju_log('ERROR', "Could not perform query. MySQL Error: %s" % (str(e)))
            cursor.close()

            if fatal is True:
                self.connection.close()
                sys.exit(1)
            else:
                return None
        else:
            if cursor.rowcount > 0:
                result = data[0]

            cursor.close()

        return result

    def db_execute(self, query, fatal=True):
        """ Perform insert/update query """
        cursor = self.connection.cursor()

        try:
            cursor.execute(query)
        except MySQLdb.Error, e:
            try:
                juju_log('ERROR', "Could not execute query. MySQL Error [%d]: %s" % (e.args[0], e.args[1]))
            except IndexError:
                juju_log('ERROR', "Could not execute query. MySQL Error: %s" % (str(e)))
            cursor.close()

            if fatal is True:
                self.connection.rollback()
                self.connection.close()
                sys.exit(1)
            else:
                return False

        return True

    def get_hostgroupid(self, name):
        """Get Host group ID by Host group name """
        groupid = self.db_fetch_one('SELECT groupid FROM groups WHERE name = "{}"'.format(name))

        return groupid

    def create_hostgroup(self, name):
        """ Create new Host group """
        groupid = self.get_nextid('groups', 'groupid')

        self.db_execute('INSERT INTO groups (groupid, name, internal) ' +
                        'VALUES ({}, "{}", 0) '.format(groupid, name))

        self.commit()

        return groupid

    def get_host_by_name(self, host_type, hostname):
        """ Get host/proxy by name using type of host """

        if 'proxy' in type:
            status = '5,6'
        else:
            status = '0,1'

        host = self.db_fetch_one('SELECT h.hostid, h.host, h.status, i.ip,i.port' +
                                 ' FROM hosts h LEFT JOIN interface i ON i.hostid = h.hostid' +
                                 ' WHERE h.status IN ({}) AND h.host = "{}" LIMIT 1'.format(status, hostname), True)

        return host

    def modify_proxy(self, hostid, service_type, hostname, ip=None, port=None):
        """ Update proxy settings like Name and ip, port in case of passive proxy by proxy ID """

        self.db_execute('UPDATE hosts SET host = "{}" WHERE hostid = {}'.format(hostid))

        if ip is not None and port is not None:
            self.db_execute('UPDATE interface SET ip = "{}", port = {} WHERE hostid = {}'.format(ip, port, hostid))

        self.commit()

        return True

    def delete_proxy(self, hostid, disable=True):
        """ Delete proxy by proxy ID and disable all related hosts with the proxy """
        if disable is True:
            self.db_execute('UPDATE hosts SET proxy_hostid = NULL, status = 1 WHERE proxy_hostid = {}'.format(hostid))
        else:
            self.db_execute('UPDATE hosts SET proxy_hostid = NULL WHERE proxy_hostid = {}'.format(hostid))

        self.db_execute('DELETE FROM hosts WHERE hostid = {}'.format(hostid))

        self.commit()

        return True

    def set_host_status(self, hostid, status=0):
        """ Update host status by host ID """
        self.db_exeucute('UPDATE hosts SET status = {} WHERE hostid = {} AND status != {}'.format(status, hostid, status))

        self.commit()

        return True

    def enable_host(self, hostid):
        """ Enable host monitoring """
        return self.set_host_status(hostid, 0)

    def disable_host(self, hostid):
        """ Disable host monitoring """
        return self.set_host_status(hostid, 1)

    def add_host(self, type, hostname, ip, port, groupid):
        # HOST_STATUS_MONITORED = 0
        status = 0

        if 'proxy-active' in type:
            # HOST_STATUS_PROXY_ACTIVE = 4
            status = 5
        if 'proxy-passive' in type:
            # HOST_STATUS_PROXY_PASSIVE = 6
            status = 6

        hostid = self.get_nextid('hosts', 'hostid')

        self.db_execute('INSERT INTO hosts (hostid, host, status, error, ipmi_username, ipmi_password, ' +
                        'ipmi_error, snmp_error, jmx_error, name, description) ' +
                        'VALUES ({}, "{}", {}, "", "", "", "", "", "", "{}", "") '.format(hostid, hostname, status, hostname))

        if groupid is not None:
            hostgroupid = self.get_nextid('hosts_groups', 'hostgroupid')

            self.db_execute('INSERT INTO hosts_groups (hostgroupid, hostid, groupid) ' +
                            'VALUES ({}, {}, {}) '.format(hostgroupid, hostid, groupid))

        if 'proxy-active' not in type:
            interfaceid = self.get_nextid('interface', 'interfaceid')

            self.db_execute('INSERT INTO interface (interfaceid, hostid, main, ip, dns, port)' +
                            'VALUES ({}, {}, 1, "{}", "", {}) '.format(interfaceid, hostid, ip, port))

        self.commit()

        return True

    def get_nextid(self, table_name, table_field):
        result = None

        nextid = self.db_fetch_one('SELECT nextid FROM ids WHERE table_name = "{}" AND field_name = "{}" FOR UPDATE'.format(table_name, table_field))

        if nextid is None:
            nextid = self.db_fetch_one('SELECT MAX({}) FROM {}'.format(table_field, table_name))

            if nextid is None:
                result = 1
                nextid = 2
            else:
                result = int(nextid) + 1
                nextid = int(nextid) + 2

            self.db_execute('INSERT INTO ids (table_name, field_name, nextid) ' +
                            'VALUES ("{}", "{}", {})'.format(table_name, table_field, nextid), False)
        else:
            result = int(result) + 1

            query_result = self.db_execute('UPDATE ids SET nextid = {} WHERE table_name = "{}" AND field_name = "{}"'.format(nextid, table_name, table_field), False)

            if query_result is False:
                self.db_execute('DELETE FROM ids WHERE table_name = "{}" AND field_name = "{}"'.format(table_name, table_field), False)

        return result

    def db_host_manage(self, action, service_type, ip, port, hostname, group_name=None):
        """ Manage host/proxy """
        host_by_name = self.get_host_by_name(service_type, hostname)

        if host_by_name is not None:
            hostid = host_by_name['hostid']
            status = int(host_by_name['status'])

            juju_log('INFO', 'The host "{}" already exists'.format(hostname))
            if action in ['changed'] and 'agent' in service_type and status == 1:
                juju_log('INFO', "Old host is disabled. Trying to enable it.")
                self.enable_host(hostid)

            if action in ['changed'] and 'agent' in service_type and intern(hostname) is not intern(host_by_name['host']):
                juju_log('INFO', 'Updating proxy name from "{}" to "{}"'.format(host_by_name['host'], hostname))
                self.modify_proxy(hostid, service_type, hostname, None, None)

            if action in ['changed'] and 'proxy-passive' in service_type and host_by_name['port'] is not None and int(host_by_name['port']) != int(port):
                juju_log('INFO', "Updating proxy network settings: IP {}:{}".format(host_by_name['ip'], port))
                self.modify_proxy(hostid, service_type, hostname, ip, port)

            if action in 'departed' and 'agent' in service_type and status == 0:
                juju_log('INFO', "Trying to disable host")
                self.disable_host(hostid)
            if action in 'departed' and 'proxy' in service_type:
                juju_log('INFO', "Deleting old proxy")
                self.delete_proxy(hostid)
        else:
            if action in ['changed'] and 'agent' in service_type:
                groupid = self.get_hostgroupid(group_name)
                if groupid is None:
                    groupid = self.create_hostgroup(group_name)
                self.add_host(service_type, hostname, ip, port, groupid)
            if action in ['changed'] and 'proxy' in service_type:
                self.add_host(service_type, hostname, ip, port, None)

        return True
